<?php

namespace Drupal\bol\Drush\Commands;

use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Drupal\Component\Serialization\Yaml;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 */
final class BolCommands extends DrushCommands {

  /**
   * Provide a bill of lading of the site's elements.
   */
    public array $types;

  #[CLI\Command(name: 'bol:scan', aliases: ['bol'])]
  #[CLI\Argument(name: 'type',description: 'A single item type to scan. Item can be all (default), content-type, paragraph-type, user-role, view, vocabulary.')]
  #[CLI\Option(name: 'csv', description: 'Send output to a CSV file instead of the screen.')]
  #[CLI\Option(name: 'disabled', description: 'Include disabled items.')]
  #[CLI\Usage(name: 'drush bol:scan', description: 'Provide a bill of lading - output to screen.')]
  #[CLI\Usage(name: 'drush bol:scan --csv', description: 'Provide a bill of lading - output to a CSV file.')]
  #[CLI\Usage(name: 'drush bol:scan --disabled', description: 'Provide a bill of lading - output only disabled items.')]
  #[CLI\FieldLabels(labels: [
    'group' => 'Group',
    'name' => 'Name',
    'type' => 'ID',
    'description' => 'Description'
  ])]
  #[CLI\DefaultTableFields(fields: ['group', 'name', 'type', 'description'])]
  #[CLI\FilterDefaultField(field: 'group')]

  public function scan($type='all', $options = ['format' => 'table', 'csv'=>false,'disabled'=>false]): RowsOfFields {
    $file = \Drupal::service('extension.path.resolver')->getPath('module', 'bol') . '/data/items.yml';
    $arr = Yaml::decode(file_get_contents($file));
    foreach ($arr['items'] as $types => $data) {
      $this->types[$types] = $data;
    }

    if ($type !== 'all' && !in_array($type, $this->types)) {
      $this->logger()->error(dt("$type is not a valid item type."));
    }

    $this->logger()->success(dt('Site scanned.'));

    $configPath = '../config/';

    if ($type == 'all') {
      $useTypes = $this->types;
    }
    else {
      $useTypes = [$type];
    }

    $all = [];
    foreach ($useTypes as $pathType => $data) {
      $ret = $this->getInfo($configPath . $this->types[$pathType]['path'], $pathType);
      if ($ret) {
        $all = array_merge($all, $ret);
      }
    }

    return new RowsOfFields($all);
  }
  public function formatOutput($list): array {
    $ret = [];
    foreach ($list as $item) {
      $ret[] = $item;
    }

    return $ret;
  }

  public function screenOutput($rows): void {
    foreach ($rows as $title => $groups) {
      print ($title . "\n");
      foreach ($groups as $group) {
        foreach ($group as $key => $value) {
          print ($key . ": " . $value) . "\n";
        }
      }
    }
  }
  private function getGlob($path): array {

    return glob($path);
  }

  private function decode($glob, $pathType): array {
    $ret = [];
    if (is_array($glob) && sizeof($glob) > 0) {
      foreach ($glob as $file) {
        $yaml = Yaml::decode(file_get_contents($file));
        $hold = [];
        if (in_array($yaml['status'], ['true', 'open'])) {
          $hold['group'] = $pathType;
          $grid = $this->types[$pathType]['grid'];
          foreach ($grid as $label => $value) {
            $hold[$label] = $yaml[$value];
          }
          $ret[] = $hold;
        }
      }
//      foreach ($list['Content Types'] as $bundle) {
//        $inventory['Fields'][] = get_list('../config/field.field.node.' . $bundle['type'] . '*', [
//          'bundle' => 'bundle',
//          'field name' => 'field_name',
//          'field type' => 'field_type',
//          'description' => 'description',
//          'label' => 'label',
//        ]);
//      }
    }

    return $ret;
  }

  public function getInfo($path, $pathType): array {
    $bol = [];
    $glob = $this->getGlob($path);
    $items = $this->decode($glob, $pathType);
    if (sizeof($items) > 0) {
      $bol = $this->formatOutput($items);
    }

    return $bol;
  }

}
